<?php
/**
 * Copyright (C) 2019 Futurenext srl
 *
 * This file is part of Zakeke.
 *
 * Zakeke Interactive Product Designer can not be copied and/or distributed
 * without the express permission of Futurenext srl
 *
 * @author    Futurenext srl <help@zakeke.com>
 * @copyright 2019 Futurenext srl
 * @license   https://portal.zakeke.com/Contracts/GeneralConditions
 */

class ZakekeInfoModuleFrontController extends ModuleFrontController
{
    /** @var int */
    protected $id_zakeke_item;

    /** @var string */
    protected $preview;

    /** @var array */
    protected $pricing;

    public function init()
    {
        parent::init();

        $id_item = Tools::getValue('id_item');

        if (!$id_item) {
            $this->errors[] = $this->module->l('id_item must be not empty.');

            return;
        }

        $id_zakeke_item = ZakekeItem::zakekeItemId($id_item);
        if ($id_zakeke_item === false) {
            $this->errors[] = $this->module->l('Zakeke item not found.');

            return;
        }

        $zakeke_item = new ZakekeItem($id_zakeke_item);

        if (!Validate::isLoadedObject($zakeke_item)) {
            $this->errors[] = $this->module->l('Zakeke item not found.');

            return;
        }

        $this->id_zakeke_item = $zakeke_item->id;
        $this->preview = $zakeke_item->preview;
        $this->pricing = Tools::jsonDecode($zakeke_item->pricing_json, true);
    }

    public function display()
    {
        ob_end_clean();
        header('Content-Type: application/json');

        if (!$this->errors) {
            $this->ajaxDie(json_encode(array(
                'success'  => true,
                'idItem'   => $this->id_zakeke_item,
                'preview'  => $this->preview,
                'pricing'  => $this->pricing
            )));
        } else {
            $this->ajaxDie(json_encode(array(
                'hasError' => true,
                'errors'   => $this->errors
            )));
        }
    }
}
